import { LogLevel } from '../types/index.js';
import { startWatch } from './watcher/index.js';
import { AuthenticatedWebSocketServer } from './connection/web-socket-server.js';
import { ModuleGraph } from './moduleGraph/index.js';
import ConsoleLogger from './consoleLogger.js';
import { Client } from './connection/client.js';
/**
 * This class represents the central place where the LWC dev server coordination occurs.
 *
 * ModuleGraph -> Is responsible for discovering modules and tracking changes to them
 * Connection -> An authenticated connection between hmr-client and dev server
 * Client -> A stateful object that tracks a hmr-client connected to the dev server
 *
 * This class also manages resources that will need clean up, like the file watcher, WebSockets.
 */
export class LWCServer {
    constructor(config, wss, logger) {
        this.clients = new Set();
        this.moduleUpdateHandler = (modulePath) => {
            for (const client of this.clients) {
                client.send({ type: 'update', data: [{ modulePath }] });
            }
        };
        // Dev Server to manage critical assets, allows them to be closed and avoid memory leaks
        this.wss = wss;
        this.watcher = startWatch(config.paths);
        this.wss.onNewConnection((connection, url) => {
            if (config.maxClientCount && this.clients.size >= config.maxClientCount) {
                logger.log(LogLevel.error, `[LWC Dev Server] Max connection limit(${this.context.config.maxClientCount}) reached, ignoring new requests until existing connections are closed`);
                connection.close();
                return;
            }
            const clientId = crypto.randomUUID();
            const client = new Client(clientId, url, connection, this.context);
            this.clients.add(client);
            client.onClose(() => {
                logger.log(LogLevel.info, `[LWC Dev Server] Closing client. Client ID: ${client.clientId}`);
                return this.clients.delete(client);
            });
            logger.log(LogLevel.info, `[LWC Dev Server] New connection for url: ${url} registered. Client ID: ${clientId}`);
            this.context.config.lifecycle?.emitTelemetry({
                eventName: 'connection-success',
                clientId: clientId,
                clientType: this.context.config.clientType ?? 'unknown',
                clientCount: this.clients.size,
            });
        });
        this.watcher.on('error', (event) => {
            this.context.logger.log(LogLevel.error, '[LWC Dev Server] Encountered error while watching files');
            this.context.logger.log(LogLevel.debug, `[LWC Dev Server] File watcher error: ${event.message}`);
            this.stopServer();
        });
        this.context = {
            config,
            logger,
            moduleGraph: new ModuleGraph(config, logger, this.watcher, {
                moduleUpdateHandler: this.moduleUpdateHandler,
            }),
        };
        this.identityToken = config.identityToken;
    }
    stopServer() {
        this.context.logger.log(LogLevel.info, '[LWC Dev Server] Stopping the dev server');
        this.watcher.close();
        this.wss.close();
        this.context.logger.log(LogLevel.info, '[LWC Dev Server] Succesfully stopped dev server');
    }
}
/**
 * Initialize an LWC Dev Server with the specified server configuration.
 * @returns Returns an initialized LWC dev server.
 */
export async function startLwcDevServer(config) {
    const logger = new ConsoleLogger(config.logLevel);
    const wss = new AuthenticatedWebSocketServer(config, logger);
    return new LWCServer(config, wss, logger);
}
//# sourceMappingURL=index.js.map